(function() {
  Ad = function() {
    this.colors_ = ['blue', 'red', 'yellow', 'blue', 'green', 'red'];
    this.colorIndex_ = 0;

    // Initialize when ready.
    if (mraid.getState() == 'loading') {
      mraid.addEventListener('ready', goog.bind(this.initialize_, this));
    } else if (document.readyState == 'loading') {
      document.addEventListener(
          'DOMContentLoaded', goog.bind(this.initialize_, this));
    } else {
      this.initialize_();
    }

    // Prevent rubber-banding.
    document.addEventListener('touchmove', this.handleTouchMove_);
    // Listen for resizing.
    window.addEventListener('resize', goog.bind(this.handleResize_, this));
  };

  Ad.prototype.expandUrl = 'interstitial.html';

  Ad.prototype.initialize_ = function() {
    // Populate references.
    this.el = {};
    this.el.banner_ = document.querySelector('#banner');
    this.el.expanded_ = document.querySelector('#expanded');
    this.el.sizeLabel_ = document.querySelector('.ruler .size');
    this.el.tileContainer_ = document.querySelector('.tiles');
    this.el.closeButton_ = document.querySelector('.close-ad');
    this.el.openWebButton_ = document.querySelector('.open-web');
    this.el.openAppButton_ = document.querySelector('.open-app');
    this.el.openOnePartExpandButton_ =
        document.querySelector('.open-one-part-expand');
    this.el.openTwoPartExpandButton_ =
        document.querySelector('.open-two-part-expand');
    this.el.slider_ = document.querySelector('.slider');

    // Listen for events.
    mraid.addEventListener(
        'stateChange', goog.bind(this.handleStateChange_, this));
    mraid.addEventListener(
        'viewableChange', goog.bind(this.handleViewableChange_, this));
    this.el.closeButton_ && this.el.closeButton_.addEventListener(
        'click', goog.bind(this.close_, this));
    this.el.openWebButton_ && this.el.openWebButton_.addEventListener(
        'click', goog.bind(this.clickToWeb_, this));
    this.el.openAppButton_ && this.el.openAppButton_.addEventListener(
        'click', goog.bind(this.clickToApp_, this));
    this.el.openOnePartExpandButton_ &&
        this.el.openOnePartExpandButton_.addEventListener('click',
        goog.bind(this.expandOnePart_, this));
    this.el.openTwoPartExpandButton_ &&
        this.el.openTwoPartExpandButton_.addEventListener('click',
        goog.bind(this.expandTwoPart_, this));

    this.el.slider_ && (this.slider_ =
        new google.ads.rich.Slider(this.el.slider_));

    this.handleResize_();
  };

  Ad.prototype.clickToApp_ = function() {
    mraid.open(
        'http://itunes.apple.com/us/app/google-mobile-app/id284815942?mt=8');
  };

  Ad.prototype.clickToWeb_ = function() {
    mraid.open('http://www.google.com/');
  };

  Ad.prototype.close_ = function() {
    mraid.close();
  };

  Ad.prototype.expandOnePart_ = function() {
    mraid.expand();
  };

  Ad.prototype.expandTwoPart_ = function() {
    mraid.expand(this.getFullUrl_(this.expandUrl));
  };

  Ad.prototype.log = function(message) {
    this.el.tileContainer_ && this.addLogTile_(message);
    this.slider_ && this.addSliderPane_(message);
  };

  Ad.prototype.addLogTile_ = function(message) {
    var tile = document.createElement('div');
    tile.className = 'tile new ' + this.getNextColor_();
    tile.innerText = message;
    this.el.tileContainer_.insertBefore(
        tile, this.el.tileContainer_.querySelector('.tile'));
    setTimeout(function() {
      goog.dom.classes.remove(tile, 'new');
    }, 0);
  };

  Ad.prototype.addSliderPane_ = function(message) {
    var pane = document.createElement('li');
    pane.className = 'message ' + this.getNextColor_();
    var text = document.createElement('span');
    text.className = 'text';
    text.innerText = message;
    pane.appendChild(text);

    var cards = this.el.slider_.children[0];
    goog.dom.classes.add(cards, 'new');
    cards.appendChild(pane);

    this.slider_.setCards(cards.children, cards.children.length - 1);

    setTimeout(function() {
      goog.dom.classes.remove(cards, 'new');
    }, 400);
  };

  Ad.prototype.getFullUrl_ = function(path) {
    if (path.indexOf('://') == -1) {
      var url = window.location.href;
      var base = url.substr(0, url.lastIndexOf('/') + 1);
      return base + path;
    } else {
      return path;
    }
  };

  Ad.prototype.getNextColor_ = function() {
    return (this.colors_[(this.colorIndex_++) % this.colors_.length]);
  };

  Ad.prototype.handleResize_ = function(e) {
    this.slider_ && this.slider_.resize(window.innerWidth);

    this.updateSizeLabel_();
    this.updateContainerSizes_();
  };

  Ad.prototype.handleTouchMove_ = function(e) {
    e.preventDefault();
  };

  Ad.prototype.handleStateChange_ = function(state) {
    state = mraid.getState();
    this.log('MRAID state: ' + state);
    if (state == 'expanded' && this.el.expanded_) {
      this.el.expanded_.style.display = 'inherit';
      this.el.banner_.style.display = 'none';
    }
    if (state == 'default' && this.el.banner_ && this.el.expanded_) {
      this.el.expanded_.style.display = 'none';
      this.el.banner_.style.display = 'inherit';
    }
  };

  Ad.prototype.handleViewableChange_ = function(isViewable) {
    this.log('MRAID viewable: ' + mraid.isViewable());
  };

  Ad.prototype.updateContainerSizes_ = function() {
    if (this.el.slider_) {
      this.el.slider_.style.width = window.innerWidth + 'px';
      this.el.slider_.style.height = window.innerHeight + 'px';
    }
  };

  Ad.prototype.updateSizeLabel_ = function() {
    this.el.sizeLabel_ && (this.el.sizeLabel_.innerText =
        window.innerWidth + 'x' + window.innerHeight);
  };

  window.google = window.google || {};
  window.google.ad = new Ad();
})();
